package mjml

import (
	"encoding/json"
	"fmt"

	"gopkg.in/olebedev/go-duktape.v3"
)

//go:generate go-bindata -pkg mjml -prefix js/dist js/dist

var mjmlSrc = `
<mjml>
  <mj-body>
    <mj-section>
      <mj-column>
        <mj-text>hello world</mj-text>
      </mj-column>
    </mj-section>
  </mj-body>
</mjml>
`

// Result ...
type Result struct {
	HTML    string
	Errors  []string
	MJML    string
	Version string
}

func initDukTapeCtx() (*duktape.Context, error) {
	ctx := duktape.New()
	js, err := mjmlHideConfigErrorJsBytes()
	if err != nil {
		return nil, err
	}
	if err := ctx.PevalString(string(js)); err != nil {
		return nil, err
	}
	js, err = bundleJsBytes()
	if err != nil {
		return nil, err
	}
	if err := ctx.PevalString(string(js)); err != nil {
		return nil, err
	}

	if err := ctx.PevalString(`
	mjml
	`); err != nil {
		return nil, err
	}
	return ctx, nil
}

// ValidationLevel for mjml rendering
type ValidationLevel string

// ValidationSkip your document is rendered without going through validation
const ValidationSkip ValidationLevel = "skip"

// ValidationSoft your document is going through validation and is rendered, even if it has errors
const ValidationSoft ValidationLevel = "soft"

// ValidationStrict your document is going through validation and is not rendered if it has any error
const ValidationStrict ValidationLevel = "strict"

// Options for mjml rendering
type Options struct {
	Beautify        bool
	Minify          bool
	ValidationLevel ValidationLevel
}

func renderDukTape(ctx *duktape.Context, mjmlSrc string, options Options) (Result, error) {
	ctx.Dup(-1)
	ctx.RequireFunction(-1)
	ctx.PushString(mjmlSrc)
	ctx.PushObject()
	ctx.PushNull()
	ctx.PutPropString(-2, "filePath")
	if options.ValidationLevel != "" {
		ctx.PushString(string(options.ValidationLevel))
		ctx.PutPropString(-2, "validationLevel")
	}
	ctx.PushBoolean(options.Beautify)
	ctx.PutPropString(-2, "beautify")
	ctx.PushBoolean(options.Minify)
	ctx.PutPropString(-2, "minify")
	ctx.Call(2)
	resultString := ctx.GetString(-1)
	ctx.Pop()
	fmt.Println(resultString)
	var result Result
	if err := json.Unmarshal([]byte(resultString), &result); err != nil {
		return result, err
	}
	return result, nil
}

/*
func initGoja() (*goja.Runtime, error) {
	vm := goja.New()
	b, err := ioutil.ReadFile("js/dist/mjml-hide-config-error.js")
	if err != nil {
		return nil, err
	}
	if _, err := vm.RunString(string(b)); err != nil {
		return nil, err
	}
	b, err = ioutil.ReadFile("js/dist/bundle.js")
	if err != nil {
		return nil, err
	}
	if _, err := vm.RunString(string(b)); err != nil {
		return nil, err
	}
	return vm, nil
}

func renderGoja(vm *goja.Runtime, mjmlSrc string) (string, error) {
	vm.Set("data", mjmlSrc)
	vm.RunString(`options={"beautify": true, "minify": false};`)
	v, err := vm.RunString("mjml(data, options);")
	if err != nil {
		return "", err
	}
	return v.Export().(string), nil
}

func main() {
	ctx := initDukTapeCtx()
	defer ctx.DestroyHeap()
	fmt.Println("duktape result is:", renderDukTape(ctx, mjmlSrc))
	fmt.Println("duktape result is:", renderDukTape(ctx, mjmlSrc))

	vm := initGoja()
	fmt.Println("goja result is:", renderGoja(vm, mjmlSrc))
	fmt.Println("goja result is:", renderGoja(vm, mjmlSrc))
}
*/
